@@ -368,6 +368,12 @@ comics_check_decompress_command	(gchar          *mime_type,
 			comics_document->command_usage = GNAUNRAR;
 			return TRUE;
 		}
+		comics_document->selected_command =
+				g_find_program_in_path ("bsdtar");
+		if (comics_document->selected_command) {
+			comics_document->command_usage = TAR;
+			return TRUE;
+		}
 
 	} else if (!strcmp (mime_type, "application/x-cbz") ||
 		   !strcmp (mime_type, "application/zip")) {
@@ -381,6 +387,12 @@ comics_check_decompress_command	(gchar          *mime_type,
 			comics_document->command_usage = UNZIP;
 			return TRUE;
 		}
+		comics_document->selected_command =
+				g_find_program_in_path ("bsdtar");
+		if (comics_document->selected_command) {
+			comics_document->command_usage = TAR;
+			return TRUE;
+		}
 
 	} else if (!strcmp (mime_type, "application/x-cb7") ||
 		   !strcmp (mime_type, "application/x-7z-compressed")) {
@@ -404,6 +416,12 @@ comics_check_decompress_command	(gchar          *mime_type,
 				comics_document->command_usage = P7ZIP;
 				return TRUE;
 			}
+			comics_document->selected_command =
+					g_find_program_in_path ("bsdtar");
+			if (comics_document->selected_command) {
+				comics_document->command_usage = TAR;
+				return TRUE;
+			}
 	} else if (!strcmp (mime_type, "application/x-cbt") ||
 		   !strcmp (mime_type, "application/x-tar")) {
 		/* tar utility (Tape ARchive) */
@@ -413,6 +431,12 @@ comics_check_decompress_command	(gchar          *mime_type,
 			comics_document->command_usage = TAR;
 			return TRUE;
 		}
+		comics_document->selected_command =
+				g_find_program_in_path ("bsdtar");
+		if (comics_document->selected_command) {
+			comics_document->command_usage = TAR;
+			return TRUE;
+		}
 	} else {
 		g_set_error (error,
 			     EV_DOCUMENT_ERROR,
@@ -957,7 +981,11 @@ extract_argv (EvDocument *document, gint page)
 					comics_document->extract_command,
 					quoted_archive,
 					quoted_filename);
+	g_free (quoted_archive);
+	g_free (quoted_filename);
+
 	g_shell_parse_argv (command_line, NULL, &argv, &err);
+	g_free (command_line);
 
 	if (err) {
 		g_warning (_("Error %s"), err->message);
@@ -965,8 +993,5 @@ extract_argv (EvDocument *document, gint page)
 		return NULL;
 	}
 
-	g_free (command_line);
-	g_free (quoted_archive);
-	g_free (quoted_filename);
 	return argv;
 }
