@@ -260,12 +260,6 @@ main (int argc, char *argv[])
 	}
 #endif
 
-#if !GLIB_CHECK_VERSION (2, 32, 0)
-	/* Init glib threads asap */
-	if (!g_thread_supported ())
-		g_thread_init (NULL);
-#endif
-
 #ifdef ENABLE_NLS
 	/* Initialize the i18n stuff */
 	bindtextdomain (GETTEXT_PACKAGE, ev_get_locale_dir());
